# Character.py

# Importion of modules as always.
import os

import pygame

import helpers
import Animation

# Initialize pygame
pygame.init()


class Character( pygame.sprite.Sprite ):
	"""
	"""

	def __init__( self, images_for_animation, animation_interval, left_top_pos, reverse_animation = False ):
		"""
		"""

		pygame.sprite.Sprite.__init__( self )

		# Let us store the parameters
		self.images_for_animation = images_for_animation
		self.animation_interval = animation_interval
		self.left_top_pos = left_top_pos
		self.reverse_animation = reverse_animation

		# Create an animation object.
		self.animation = Animation.Animation( self.images_for_animation, self.animation_interval, self.reverse_animation )

		# Create an animation object for the images.
		self.jump_image = helpers.load_image( "jump_3.png" )
		self.jump_image = pygame.transform.smoothscale( self.jump_image, [ 80, 80 ] )

		# Create the sprite.
		self.image = self.animation.image
		self.image.set_colorkey( ( 0, 113, 188 ) )
		self.rect = self.image.get_rect()
		self.rect.topleft = self.left_top_pos


		# Properties for the jump
		self.jumping = False
		self.jump_max_height_reached = False
		self.jump_max_height = 90
		self.jump_height_interval = 2
		self.jump_current_height = 0
		self.jump_acc = 4

		self.double_jumping = False


		# Properties for if a player has fallen through a hole.
		self.has_fallen_through_hole = False


		# Crates
		self.was_on_crate = False

		# Sound
		self.running_sound = pygame.mixer.Sound( os.path.join( "../sounds/untitled54779__bevangoldswain__running-hard-surface2.wav" ) )
		self.running_sound_already_playing = False
		

	def fall_through_hole( self ):
		"""
		This method lets the character fall through a hole.
		"""

		if self.has_fallen_through_hole:
			# 220 == display height.
			if self.rect.top > 220:
				self.has_fallen_through_hole = False
				return True
			else:
				self.rect.move_ip( 0, 10 )
				return False
				
		return False


	def update( self, eventlist ):
		"""
		"""

		if not self.jumping and not self.running_sound_already_playing:
			self.running_sound.play( -1 )
			self.running_sound_already_playing = True
			
		if self.has_fallen_through_hole:

			if self.fall_through_hole():
				return True


		# Check for any keys.
		for event in eventlist:
			if event.type == pygame.KEYDOWN:
				if event.key == pygame.K_SPACE and not self.jumping:
					self.jump_has_started()
				elif event.key == pygame.K_SPACE and self.jumping and not self.double_jumping:
					self.double_jump_has_started()

		if self.jumping:
			# Stop playing the running sound.
			self.running_sound_already_playing = False
			self.running_sound.stop()
			
			self.image = self.jump_image
			self.jump()
		else:
			# Update the animation object.
			if self.animation.update():
				self.image = self.animation.image




		return False

	def double_jump_has_started( self ):
		"""
		"""

		self.double_jumping = True
		if self.jump_max_height_reached:
			self.jump_max_height_reached = False
			self.jump_acc = 0
			self.jump_acc = 3
		else:
			self.jump_acc = 0
			self.jump_acc = 3
		
	def jump_has_started( self ):
		"""
		Raises our character into the air.
		"""

		self.jumping = True


	def jump( self ):
		"""
		"""

		if not self.jump_max_height_reached:
			self.jump_current_height += self.jump_acc
			self.rect.move_ip( 0, self.jump_acc * - 1 )
			self.jump_acc -= 0.1


			if self.jump_acc <= 0:
				self.jump_max_height_reached = True
				self.jump_acc = 0

		elif self.jump_max_height_reached:
			self.jump_current_height -= self.jump_acc
			self.rect.move_ip( 0, self.jump_acc )

			if self.rect.bottom >= 220: #self.jump_acc >= 4.00:
				self.jump_max_height_reached = False
				self.jumping = False
				self.jump_acc = 4
				self.rect.top = 120
				self.double_jumping = False
			self.jump_acc += 0.1				


def character_has_collided( character_sprite, path_sprites ):
	"""
	"""

	collided_sprites = pygame.sprite.spritecollide( character_sprite.sprite, path_sprites, False )

	for path in collided_sprites:
		if path.type_of_tile == 2:
			if  ( ( character_sprite.sprite.rect.left - 2 ) > path.rect.left )  and ( character_sprite.sprite.rect.right < ( path.rect.right + 5 ) ):
				character_sprite.sprite.has_fallen_through_hole = True # CHANGE TO TRUE LATER
				return False
				
		elif path.type_of_tile == 3:
			if character_sprite.sprite.rect.right > path.rect.left + 40 and character_sprite.sprite.rect.left < path.rect.right - 40 and character_sprite.sprite.rect.bottom > path.rect.top + 30:
				return True # CHANGE TO TRUE LATER
				
		elif path.type_of_tile is not 1:
			if character_sprite.sprite.rect.right > path.rect.left + 30 and character_sprite.sprite.rect.bottom > path.rect.top + 30:
				return True
			#if character_sprite.sprite.rect.bottom > path.rect.centery:
			#	return True

	return False


#  Functions.
def get_character_animation_images():
	"""
	This function returns the images for our character animation.

	Return value
		pygame.Surface List
			The list containing the images in the correct order.
	"""

	amoumnt_of_images = 6
	images = []

	for x in range( 1, 6 ):
		image = helpers.load_image( "runner_" + str( x ) + ".png" )
		image = pygame.transform.smoothscale( image, [ 80, 80 ] )
		images.append( image )
		
	return images


def get_character_jump_animation_images():
	"""
	This function returns a list of images used for our characters jumping animation.

	Return value
		pygame.Surface list
			The list containing the images in the correct order.
	"""

	amount_of_images = 5
	images = []

	for x in range( 1, amount_of_images ):
		image = helpers.load_image( "jump_" + str( x ) + ".png" )
		image = pygame.transform.smoothscale( image, [ 100, 100 ] )
		images.append( image )

	return images
